@@ -10,10 +10,10 @@ Module for collecting data to be sent to the developer.
 
 
 import logging
+import os
 
 from . import dev_classes
 from . import utils
-from .compat import os_ as os
 from .message import Message
 
 
@@ -34,16 +34,16 @@ def main(args, argv):
     function.
     """
     setupDevLogger(args.config_path, args.log)
-    currentdir = os.getcwdu() # Store this just in case the paths that have been given are relative
+    currentdir = os.getcwd() # Store this just in case the paths that have been given are relative
     if args.out_path:
         if not os.path.exists(args.out_path):
             os.makedirs(args.out_path)
         out = args.out_path
     else:
         out = currentdir
-    logger.log(5, 'ARGV: {}'.format(argv))
+    logger.log(5, f'ARGV: {argv}')
     for y, x in enumerate(args.msgs):
-        logger.log(5, '---- RUNNING DEVELOPER MODE ON FILE {} ----'.format(x[0]))
+        logger.log(5, f'---- RUNNING DEVELOPER MODE ON FILE {x[0]} ----')
         logger.log(5, 'EXCEPTION CHECK:')
         try:
             with Message(x[0]) as msg:
@@ -64,4 +64,4 @@ def main(args, argv):
                 logpath = x.baseFilename
             except AttributeError:
                 pass;
-        print('Logging complete. Log has been saved to {}'.format(logpath))
+        print(g'Logging complete. Log has been saved to {logpath}')
